/*
 * TestStage4.cpp
 *
 *  Created on: 24-07-2019
 *  Author: Matias Contreras, Camilo Fernandez, Maria Rivas
 */

#include "SemaforoP.h"
#include "SemaforoDeGiro.h"
#include "StreetTrafficLight.h"
#include "DetectorDeRequerimiento.h"
#include "SimuladorEntradas.h"
#include "Controlador.h"
#include "MyTimer.h"

#include <chrono>
#include <thread>
#include <iostream>			// Biblioteca para la entrada y salida de datos
#include <fstream>			// Biblioteca para el tratamiendo de archivos
#include <unistd.h>			// Biblioteca para utilizar el metodo sleep()
#include <string.h>			// Biblioteca para el uso de cadenas de caracteres
using namespace std;

int main(int argc, char *argv[]){
	ifstream file;
	string data, aux, out;

	// Times Street Traffic Lights
	int followTimeStreet = 7;
	int transitionTimeStreet = 3;
	int stopTimeStreet = followTimeStreet + transitionTimeStreet;

	// Times Pedestrian Traffic Lights 
	int followTimeWalker = 5;
	int transitionTimeWalker = 5;

	// Turn Traffic Lights
	int followTimeTurn = 6;
	int transitionTimeTurn = 4;

	if(argc != 2){
		cout << "Usage: ./TestStage4 <name of event input file>\n" << endl;
		return 1;
	}

	file.open(argv[1]);
	if(file.fail()){
		cout << "Error when opening file\n" << endl;
		return 1;
	}

	while(getline(file,aux))		// Lee linea por linea desde file
		out.append(aux);			// Las almacena en un string

	file.close();

	int a = 0;
	char line[out.length()];
	for(int i = 0; i<out.length(); i++){				// Detecta lineas y espacios vacios para
		if((out[i] != '\r') && (out[i] != '\t')){ 		// eliminarlos de out y almacenarlos en data
			line[a] = out[i]; a++; 
		}		
	}

	for(int i = 0; i<strlen(line); i++){
		if(line[i] == '1' || line[i] == '0')
			data.push_back(line[i]);
	}
		

	// Inductive Sensor and Button
	DetectorDeRequerimiento *sensor = new DetectorDeRequerimiento();
	DetectorDeRequerimiento *button = new DetectorDeRequerimiento();

	// Pedestrian Traffic Lights
	SemaforoP *A = new SemaforoP(followTimeWalker, transitionTimeWalker);
	SemaforoP *B = new SemaforoP(followTimeWalker, transitionTimeWalker);

	// Turn Traffic Lights
	SemaforoDeGiro *two = new SemaforoDeGiro(followTimeTurn, transitionTimeTurn);
	SemaforoDeGiro *four = new SemaforoDeGiro(followTimeTurn, transitionTimeTurn);

	// Street Traffic Lights
	StreetTrafficLight *one = new StreetTrafficLight(stopTimeStreet, transitionTimeStreet, followTimeStreet);
	StreetTrafficLight *three = new StreetTrafficLight(stopTimeStreet, transitionTimeStreet, followTimeStreet);
	StreetTrafficLight *five = new StreetTrafficLight(stopTimeStreet, transitionTimeStreet, followTimeStreet);

	// Controller, Simulator and Timer
	Controlador *controller = new Controlador(A, B, one, two, three, four, five, sensor, button);
	SimuladorEntradas *simulator = new SimuladorEntradas(sensor, button, data);
	MyTimer *timer = new MyTimer(500, simulator);

	cout << " \tA\tB\t1\t2\t3\t4\t5" << endl;
	simulator->actionPerformed();
	for(int i = 0; i<data.length(); i++)
		controller->manageTraffic();
	

	timer->stop();
	delete sensor, button, A, B, one, two, three, four, five, controller, simulator, timer;
	return 0;
}
